--[[
**********************************************************************
MagicDKP - a raid loot and DKP tracker.
**********************************************************************
This file is part of MagicDKP, a World of Warcraft Addon

MagicDKP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MagicDKP is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]

--[[ Config dummy

_[Settings]_[Items]_[Players]_[Raids]_[Events]_________


Items =>
   Item Name
     Cost => Item Cost
     Type => Ignore / Bank / Fixed Cost / Variable Cost
     Item Link => Link

Players =>
  Name =>
    Level
    Class
    Guid
    Race
    Alts => [a, b, c]

Raids =>
  RaidDate/Name =>
    Start, End => Start/end of raid
    OnTime =>
       attendees = { PlayerName },
       standby = {}
    BossAttempt =>

 ]]

local MagicDKP = MagicDKP
local L = LibStub("AceLocale-3.0"):GetLocale("MagicDKP", false)
local R = LibStub("AceConfigRegistry-3.0")
local C = LibStub("AceConfigDialog-3.0")
local DBOpt = LibStub("AceDBOptions-3.0")

local BabbleZone = LibStub("LibBabble-Zone-3.0") 
local ZR = BabbleZone:GetReverseLookupTable()
local ZL  = BabbleZone:GetLookupTable()

local BabbleBoss = LibStub("LibBabble-Boss-3.0") 
local BR = BabbleBoss:GetReverseLookupTable()
local BL = BabbleBoss:GetUnstrictLookupTable()

local options, cmdoptions, standardZoneOptions, standardBossOptions, standardEventOptions, standardRaidOptions, stdEventOptions
local logLevelsDropdown = {}
local zones, db

local idmatch = "(%a+):(%d+)"

for logname,id in pairs(MagicDKP.logLevels) do
   logLevelsDropdown[id] = L[logname]
end

BabbleZone = nil

local function UpdateUpvalues()
   db = MagicDKP.db.profile         -- profile data
   zones = MagicDKPDB.zones     -- raid zones / boss mobs
   items = MagicDKPDB.items     -- item database
   events = MagicDKPDB.events   -- custom events
   raids = MagicDKPDB.raids     -- raids
   players = MagicDKPDB.players -- player db
end

local function GetBossName(arg)
   local region = arg[#arg-2]
   local name =  zones[ region ] and zones[ region ].mobs[ tonumber(arg[#arg-1]) ].name
   return name and  BL[name] or name
end

local function highlite(str)
   return string.format("|cff33ff99%s|r", tostring(str))
end

local function GetItemDesc(arg)
   local itemid
   local var,num
   if type(arg) == "table" then
      var, num = string.match(arg[#arg], idmatch)
   end
   itemid = (num and tonumber(num)) or (type(arg) == "table" and tonumber(arg[#arg-1])) or tonumber(arg)
   return itemid..":"..items[itemid].itemInfo[2]
end

local function GetEventName(info)
   return info[#info-1]
end
   
local function GetEventDesc(info)
   local event = info[#info-1]
   local raid = tonumber(info[#info-2])

   return MagicDKP:FromUnixTime(raids[raid].events[event].eventTime, L["Event Time"]..": |cffffffff", "|r")
end

local function GetRaidName(raidid)
   local name = raids[raidid].zone
   return MagicDKP:FromUnixTime(endTime and raids[raidid].endTime or raids[raidid].startTime,
				raidid..":", name and ZL[name], name)
end


local function GetRaidDesc(info)
   local raid = tonumber(info[#info-1])
   local desc
   desc = MagicDKP:FromUnixTime(raids[raid].startTime, L["Raid Start Time"]..": |cffffff50", "|r")
   if raids[raid].endTime then
      desc = desc .. "\n"..MagicDKP:FromUnixTime(raids[raid].endTime, L["Raid End Time"]..": |cffffff50", "|r")
   else
      desc = desc .."\n".. L["Raid End Time"]..": |cffffff50".. L["Still in progress"].. "|r"
   end
   return desc
end

local function GetItemTexture(arg)
   local var,num = string.match(arg[#arg], idmatch)
   local itemid = tonumber(arg[#arg-1])
   if var == "name" and num then
      itemid = tonumber(num)
   end
   return items[itemid].texture
end

function MagicDKP:ListAlts(_,alt)
   local alts = {}
   if not alt or not players[alt] then 
      if self.error then self:error("Invalid parameters: Expected a valid player name.") end
      return
   end
   alt = alt:lower():trim():gsub("^.", string.upper)
   for name,data in pairs(players) do
      if data.main == alt then
	 alts[#alts+1] = name
      end
   end
   if #alts > 0 then
      table.sort(alts)
      self:Print(string.format("Alts registered for %s: %s", alt, table.concat(alts, ", ")))
   else
      rself:Print(string.format("There are no alts registered for %s", alts))
   end
end

function MagicDKP:AssociateAlt(alt, player)
   if not player or not alt then
      if self.error then self:error("Invalid parameters: Expected [alt player].") end
      return
   end
   alt = alt:lower():trim():gsub("^.", string.upper)
   player = player:lower():trim():gsub("^.", string.upper)
   if not players[player] then
      if self.error then self:error("Unknown player: %s", player) end
      return
   end
   if not players[alt] then
      self:UpdatePlayerInfo(alt, "Unknown", 0, "Unknown")
   end
   if players[player].main then
      if self.error then self:error("Player %s is already registered as an alt for %s.", player, players[player].main) end
      return
   end
      
   players[alt].main = player
   if self.info then self:info("Registered <%s> as an alt for <%s>.",  alt, player) end
end

local cmdoptions = {
   type = "group",
   name = L["Magic DKP"],
   handler = MagicDKP,
   args = {
      ["additem"] = {
	 type = "input",
	 name = L["args: <looter> <itemlink> - add an item to the current raid/event"],
	 set = "AddItemManually",
	 get = function()end,
      },
      ["alt"] = {
	 type = "group",
	 name = L["Alternate character management."],
	 args = {
	    add = {
	       type = "input",
	       name = L["alt add <alt> <player> - associate <alt> to <player>"],
	       set = function(info, val) MagicDKP:AssociateAlt(strsplit(" ", val)) end,
	       get = function() end,
	    },
	    list = {
	       type = "input",
	       name =  L["alt list <player> - list all alts of <player>"],
	       set = "ListAlts",
	       get = function() end,
	    },
	    del = {
	       type = "input",
	       name = L["alt clear <alt> - remove the alt association for <alt>"],
	       set = function(info, val) if players[val] then
			   players[val].main = nil
			   if MagicDKP.info then MagicDKP:info("%s is no longer registered as an alt.", val) end
			else
			   if MagicDKP.error then MagicDKP:error("Unknown player: %s", val) end
			end
		     end,
	       get = function() end,
	    },
	 }
      },
      ["end"] = {
	 type = "execute",
	 name = L["Force the current active raid, if any, to end."],
	 func = function()
		   if not db.enabled then MagicDKP:Print("Addon is not enabled. Use /magicdkp toggle to enable it.") return end
		   MagicDKP:EndRaid()
		   MagicDKP:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
		   MagicDKP:UnregisterEvent("PLAYER_REGEN_DISABLED")
		   MagicDKP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")   
		   MagicDKP:UnregisterEvent("PLAYER_TARGET_CHANGED")
		   MagicDKP:UnregisterEvent("CHAT_MSG_LOOT")
		   MagicDKP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
      },
      ["start"] = {
	 type = "execute",
	 name = L["Force the current raid to end and a new one to be created."],
	 func = function()
		   if not db.enabled then MagicDKP:Print("Addon is not enabled. Use /magicdkp toggle to enable it.") return end
		   MagicDKP:EndRaid() -- end old raid if present
		   MagicDKP:FindOrCreateRaid() -- create a new one
		   MagicDKP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "ScanTarget", "mouseover")   
		   MagicDKP:RegisterEvent("PLAYER_TARGET_CHANGED", "ScanTarget", "target")   
		   MagicDKP:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		   MagicDKP:RegisterEvent("CHAT_MSG_LOOT")	    
		   MagicDKP:RegisterEvent("PLAYER_REGEN_DISABLED")
		   MagicDKP:ZONE_CHANGED_NEW_AREA() -- trigger zone change logic to run
		end,
      },
      ["toggle"] = {
	 type = "toggle",
	 name = L["Enable or disable the addon."],
	 set = function()
		  db.enabled = not db.enabled
		  if db.enabled then
		     MagicDKP:OnEnable()
		     MagicDKP:Print("Addon is now enabled.")
		  else
		     MagicDKP:OnDisable()
		     MagicDKP:Print("Addon is now disabled.")
		  end
	       end,
	 get = function() return db.enabled end,
      },
      versions = {
	 type = "execute",
	 name = L["Query raid for their MagicDKP versions."],
	 func = "QueryAddonVersions",
      },
      config = {
	 type = "execute",
	 name = L["Toggle configuration dialog"],
	 func = "ToggleConfigDialog",
      },
      wipe = {
	 type = "execute",
	 name = L["Log a wipe for the current boss"],
	 func = "AddWipe",
      },
      event = {
	 type="group",
	 name = L["Take a custom event snapshot."],
	 args = {
	 }
      }
      
   }
}
	 
local options = {
   type = "group", 
   name = L["Magic DKP"],
   childGroups = "tab",
   handler = MagicDKP,
   args = {
      settings = {
	 type = "group",
	 name = L["Settings"],
	 set = "SetProfileParam",
	 get = "GetProfileParam",
	 args = {
	    guildranks = {
	       type = "group",
	       name = L["Guild Ranks"],
	       order = 2,
	       args = {
		  header = {
		     type = "header",
		     name = L["Guild Ranks"],
		     order = 0
		  },
		  help = {
		     type = "description",
		     name = L["Select the raiding ranks of your guild. Only the selected ranks will be included in the raid standby calls."],
		     order = 1,
		  },
		  ranks = {
		     type = "multiselect",
		     name = "",
		     values = "GetGuildRanks", 
		     get = "GetGuildRank",
		     set = "SetGuildRank",
		  }		  
	       }
	    },
	    options = {
	       type = "group",
	       name = L["Options"],
	       order = 1,
	       args = {
		  enabled = {
		     type = "toggle",
		     name = L["Enable or disable the addon."],
		     desc = L["ENABLE EVENTS"],
		     width = "full",
		     set = function()
			      db.enabled = not db.enabled
			      if db.enabled then
				 MagicDKP:OnEnable()
				 MagicDKP:Print("Addon is now enabled.")
			      else
				 MagicDKP:OnDisable()
				 MagicDKP:Print("Addon is now disabled.")
			      end
			   end,
		  },	
		  standbyattendee = {
		     type = "toggle",
		     name = L["Add Standby to Raid"],
		     desc = L["ADD STANDBY HELP"],
		     order = 25
		  },
		  eventnameaszone = {
		     type = "toggle",
		     name = L["Use custom event name as zone name"],
		     desc = L["CUSTOM EVENT ZONE NAME"],
		     width = "full",
		     order = 25
		  },
		  logLevel = {
		     type = "select",
		     name = L["Log level"],
		     desc = L["LOGLEVELHELP"],
		     values = logLevelsDropdown,
		     order = 10,
		  },
		  raidsize  = {
		     type = "select",
		     name = L["Minimum Raid Size"],
		     desc = L["MIN RAID HELP"], 
		     values = {
			["25"] = L["25-Man"],
			["10"] = L["10-Man"],
			["2"]  = L["Any size raid"],
		     },
		     order = 20,
		     
		  },
		  trackwipe = {
		     type = "toggle",
		     name = L["Track wipes as events"],
		     desc = L["TRACK WIPE HELP"],
		     order = 30,
		     hidden = true,
		  },
		  wipepercentage = {
		     type = "range",
		     name = L["Wipe Percentage"],
		     desc = L["WIPE PERCENTAGE HELP"],
		     min = 1, max = 100, step = 1,
		     order = 40,
		     hidden = true,
		     disabled = function() return not db.trackwipe end,
		  },
	       }
	    },
	 }, 
	 order = 1,
      }, 
      zones = {
	 type = "group",
	 name = L["Raid Zones"],
	 args = {}, 
	 order = 2,
      }, 
      events = {
	 type = "group",
	 name = L["Events"],
	 set = "SetEventName",
	 get = "GetEventName",
	 args = {
	    addcustom = {
	       type = "group",
	       name = L["Add custom event"],
	       args = {
		  eventname = {
		     type = "input",
		     name = L["Event name"],
		     order = 1,
		     set = "SetAddCustomEvent",
		     get = "GetAddCustomEvent",
		  },
		  addevent = {
		     type = "execute",
		     name = L["Add event"],
		     order = 1000,
		     func = "AddCustomEvent"
		  },
	       },
	       order = 100000,
	    },
	 }, 
	 order = 3,
      }, 
      itemdb = {
	 type = "group",
	 name = L["Item database"],
	 args = {
--	    add = {
--	       type = "group",
--	       name = L["Add Item..."],
--n	       order = 0,
--	       args = {
--	       }
--	    }
	 }, 
	 order = 4,
      }, 
      playerdb = {
	 type = "group",
	 name = L["Player database"],
	 args = {}, 
	 order = 5,
      }, 
      raids = {
	 type = "group",
	 name = L["Raids"],
	 args = {}, 
	 order = 6,
      }, 
   }
}
    
standardBossOptions = {
   header = {
      name = GetBossName,
      type = "header",
      order = 0
   },
   isfarm = {
      name = L["Is farm"],
      type = "toggle",
      desc = L["FARM HELP"],
      order = 2,
   },
   ignore = {
      name = L["Ignore boss"],	
      type = "toggle",
      desc = L["IGNORE HELP"],
      order = 1, 
   },
   event = {
      name = L["Event name"],
      type = "input",
      desc = L["EVENT NAME HELP"],
      order = 10,
   }, 
   resetevent = {
      name = L["Reset event name"],
      type = "execute",
      desc = L["RESET EVENT NAME HELP"],
      order = 12,
      func = "ResetBossEvent",
      hidden = "IsDefaultBossEvent",
   }, 
   notes = {
      name = L["Notes"],
      type = "input",
      multiline = true,
      order = 100,
      width = "full",
   }
}

standardEventOptions = {
   header = {
      name = GetEventName,
      type = "header",
      order = 0
   },
--   value = {
--      name = L["Event Value"],
--      type = "input",
--      desc = L["EVENT VALUE HELP"],
--      order = 10,
--   }, 
   eventInfo = {
      type = "description",
      name = GetEventDesc,
      order = 2
   },
}

standardRaidOptions = {
   header = {
      name = function(info) return GetRaidName(tonumber(info[#info-1]))end,
      type = "header",
      order = 0
   },
   raidInfo = {
      type = "description",
      name = GetRaidDesc,
      order = 2
   },
   export = {
      type = "execute",
      name = L["CTRT Export"],
      desc = L["Export raid information in a format understood by CT_RaidTracker compatible importers."],
      func = "ExportCTRT",
   }
}


stdEventOptions = {
   type = "group",
   name = function(info) return info[#info] end,
   args = {
      eventname = {
	 type = "input",
	 name = L["Event name"],
	 order = 1
      },
      delevent = {
	 type = "execute",
	 name = L["Delete"],
	 order = 2
      },
   }
}

local cmdEventOption = {
   type = "execute",
   name = L["Take attendance for this event"],
   func = function(info) MagicDKP:LogCustomEvent(info[#info]) end
}

standardItemOptions = {
   iteminfo = {
      type = "input",
      name = GetItemTexture,
      dialogControl = "MDKPItemWidget",
      width = "full",
      order = 0,
      get = GetItemDesc,
   },
   ignore = {
      name = L["Ignore"],	
      type = "toggle",
      desc = L["IGNORE ITEM HELP"],
      order = 1, 
   },
   bank = {
      name = L["Bank Item"],
      type = "toggle",
      desc = L["BANK ITEM HELP"],
      disabled = "IsItemIgnored",
      order = 2, 
   }, 
   type = {
      name = L["Fixed Value"],
      type = "toggle",
      desc = L["FIXED VALUE HELP"],
      disabled = "IsItemUnvalued",
      order  = 4,
   }, 
   value = {
      name = L["Value"],
      type = "input",
      desc = L["VALUE HELP"],
      disabled = "IsItemUnvalued",
      order = 3,
   },
}

do
   local tmpCustomEvent
   function MagicDKP:SetAddCustomEvent(info,val) tmpCustomEvent = val end
   function MagicDKP:GetAddCustomEvent(info) return tmpCustomEvent end
   function MagicDKP:AddCustomEvent(info)
      if not tmpCustomEvent then return end
      local key = self:MakeKey(tmpCustomEvent)
      if strlen(key) > 0 then
	 key = strlower(key)
	 events[key] = tmpCustomEvent
	 options.args.events.args[key] = stdEventOptions
	 cmdoptions.args.event.args[key] = cmdEventOption
	 self:NotifyChange()
      end
      tmpCustomEvent = nil
   end
end


do 
   local simpleNameCache = {}
   function MagicDKP:MakeKey(name)
      if not name then return "" end
      if not simpleNameCache[name] then
	 simpleNameCache[name] = gsub(name, " ", "")
      end
      return simpleNameCache[name]
   end
end


function MagicDKP:GetGuildRanks()
   local ranks = {}
   for id = 1, GuildControlGetNumRanks() do
      ranks[string.format("%02d", id)] = string.format("%d: %s", id, GuildControlGetRankName(id))
   end
   return ranks
end
function MagicDKP:GetGuildRank(info, key)
   key = tonumber(key)
   if db.ranks then
      return db.ranks[key]
   end
end
function MagicDKP:SetGuildRank(info, key, state)
   key = tonumber(key)
   if not db.ranks then db.ranks = {} end
   if state then
      db.ranks[key] = state
   else
      db.ranks[key] = nil
   end
end


function MagicDKP:BuildZoneConfig(var)
   local zone = (type(var) == "table" and var[#var-1]) or var
   local zoneHash = options.args.zones.args[zone]

   if self.trace then self:trace("Generating configuration for mobs in zone %s", zoneHash.name) end

   zoneHash.args.loader.hidden = true
   
   for mob,data in pairs(zones[zone].mobs) do
      if self.spam then self:spam("  Added %s...", data.name) end
      mob = tostring(mob)
      if not zoneHash.args[mob] then 
	 zoneHash.args[mob] = {
	    type = "group",
	    name = BL[data.name] or data.name,
	    args =  standardBossOptions,
	    get = "GetBossOption",
	    set = "SetBossOption",
	    --	 plugins =  {
	    --	    extras = {
	    --	       type = 'execute',
	    --	       name = L['Delete mob from database (not recoverable)'],
	    --	       order = 10001,
	    --	       width = "full",
	    --	       func = "RemoveMob",
	    --	       confirm = true,
	    --	       confirmText = string.format(L["Are you sure you want to delete |cffd9d919%s|r from the database?"], BL[data.name])
	    --	    }
	    --      }
	 }
      end
   end
   self:NotifyChange()
end
 
local tmpsorttable = {}
local function GetSortedList(attend)
   for id in pairs(tmpsorttable) do
      tmpsorttable[id] = nil
   end
   if attend then
      for id in pairs(attend) do
	 local pinfo = players[id] 
	 if pinfo and pinfo.main then
	    id = string.format("%s (%s)", pinfo.main, id)
	 end
	 tmpsorttable[#tmpsorttable+1] = id
      end
      table.sort(tmpsorttable)
      return table.concat(tmpsorttable, ", ")
   end
   return "None"
end
local function GetRaidAttendees(info)
   local event = info[3]
   local raidid = tonumber(info[2])
   if MagicDKP.spam then MagicDKP:spam("GetRaidAttendees(%s, %s)", tostring(raidid), tostring(event)) end
   return GetSortedList(raids[raidid].events[event].attendees)
end

local function GetRaidStandby(info)
   local event = info[3]
   local raidid = tonumber(info[2])
   if MagicDKP.spam then MagicDKP:spam("GetRaidStandby(%s, %s)", tostring(raidid), tostring(event)) end
   return GetSortedList(raids[raidid].events[event].standby)
end

local attendeeConfig = {
   attendHeader = {
      type = "header",
      name = L["Attendees"],
      order = 50,
   },
   attendList = {
      type = "description",
      name = GetRaidAttendees,
      order = 51,
   },
   
   standbyHeader = {
      type = "header",
      name = L["Standby Attendees"],
      order = 100,
   },
   standbyList = {
      type = "description",
      name = GetRaidStandby,
      order = 101,
   },
}

function MagicDKP:BuildRaidConfig(var)
   local raidid = (type(var) == "table" and var[#var-1]) or var
   local raidHash = options.args.raids.args[raidid]
   
   if self.trace then self:trace("Generating configuration for raid %d", raidid) end

   raidHash.args.loader.hidden = true
      
   for event,data in pairs(raids[tonumber(raidid)].events) do
      if self.spam then self:spam("  Added event %s...", event) end
      if not raidHash.args[event] then 
	 raidHash.args[event] = {
	    type = "group",
	    name = event,
	    get = "GetEventOption",
	    set = "SetEventOption",
	    order = data.eventTime,
	    args = {
	    },
	    plugins = {
	       participants = attendeeConfig,
	    }
	 }
      end
      for key, val in pairs(standardEventOptions) do
	 raidHash.args[event].args[key] = val
      end

      if data.items then
	 raidHash.args[event].args.items =  {
	    type = "group",
	    name = L["Items"],
	    args = {}
	 }
	 local subhash = raidHash.args[event].args.items.args
	 local looterHash = {
	    ["00BANK"] = L["Bank"],
	    ["00DE"] = L["Disenchant"],
	 }
	 for val in pairs(data.attendees) do
	    looterHash[val] = val
	 end
	 
	 for id,val in pairs(data.items) do
	    local order = id * 10
	    subhash["header:"..id] = {
	       type = "header",
	       name = "",
	       order = order
	    }
	    subhash["name:"..val.itemid..":"..id] = {
	       type = "input",
	       name = GetItemTexture,
	       dialogControl = "MDKPItemWidget",
	       width = "full",
	       get = GetItemDesc,
	       order = order +1 
	    }
	    subhash["looter:"..id] = {
	       type = "select",
	       name = L["Looter"],
	       order = order +2,
	       values = looterHash,
	       get = "GetLootOption",
	       set = "SetLootOption",
	    }
	    subhash["value:"..id] = {
	       type = "input",
	       name = L["Cost"],
	       order = order + 3,
	       get = "GetLootOption",
	       set = "SetLootOption",
	    }
	    --	 subhash["count:"..id] = {
	    --	    type = "range",
	    --	    name = L["Count"],
	    --	    order = order + 3,
	    --	    min = 1, max = 5, step = 1,
	    --	    get = "GetLootOption",
	    --	    set = "SetLootOption",
	    --	 }
	 end
      end
   end
   self:NotifyChange()
end


function MagicDKP:BuildConfiguration()
   local subopts

   UpdateUpvalues()


   -- Custom events
   subopts = options.args.events.args
   for id, event in pairs(events) do
      subopts[id] = stdEventOptions
      cmdoptions.args.event.args[id] = cmdEventOption
   end
   
   
   -- Set up raid zones
   subopts = options.args.zones.args
   for zone, data in pairs(zones) do
      if not subopts[zone] then 
	 local name = data.name 
	 if name == "World Boss" then
	    name = L["World Boss"]
	 else
	    name = ZL[name]
	 end
	 subopts[zone] = {
	    type = "group",
	    name = name, 
	    args = {
	       loader = {
		  name = "Loader",
		  type = "toggle",
		  get = "BuildZoneConfig", 
		  set = function() end
	       }
	    }
	 }
      end
   end


   for raidid = 1, #raids do
      self:AddRaidConfig(raidid)
   end

   for item,_ in pairs(items) do
      self:AddItemConfig(item)
   end
   -- Profile handling
   options.args.settings.args.profile = DBOpt:GetOptionsTable(MagicDKP.db)

end

function MagicDKP:DeleteRaidConfig(raidid)
   options.args.raids.args[tostring(raidid)] = nil
end

function MagicDKP:AddRaidConfig(raidid)
   local subopts = options.args.raids.args
   local raidkey = tostring(raidid)
   if not subopts[raidkey] or subopts[raidkey].args.loader.hidden then
      subopts[raidkey] = {
	 type = "group",
	 name = GetRaidName(raidid),
	 order = -raidid,
	 args = {
	    loader = {
	       name = "Loader",
	       type = "toggle",
	       get = "BuildRaidConfig", 
	       set = function() end
	    },
	 }
      }
      for key,val in pairs(standardRaidOptions) do
	 subopts[raidkey].args[key] = val
      end
   end
end

function MagicDKP:ToggleConfigDialog()
   if C.OpenFrames["Magic DKP"] then
      C:Close("Magic DKP")
   else
      C:Open("Magic DKP")
      self:SetStatusText(string.format(L["Active profile: %s"], self.db:GetCurrentProfile()))
   end
end

do
   local updateStatusTimer
   function MagicDKP:SetStatusText(text, update)
      local frame = C.OpenFrames["Magic DKP"]
      if frame then
	 frame:SetStatusText(text)
	 if updateStatusTimer then self:CancelTimer(updateStatusTimer, true) end
	 if update then
	    updateStatustimer = self:ScheduleTimer("SetStatusText", 10, string.format(L["Active profile: %s"], self.db:GetCurrentProfile()))
	 else
	    updateStatustimer = false 
	 end
      end
   end
end

function MagicDKP:NotifyChange()
   UpdateUpvalues()
   R:NotifyChange(L["Magic DKP"])
end
 
function MagicDKP:SetProfileParam(var, value)
   local varName = var[#var]
   db[varName] = value
   if self.spam then 
      self:spam("Setting parameter %s to %s.", varName, tostring(value))
   end

   if varName == "logLevel" then
      self:SetLogLevel(value)
   elseif varName == "raidsize" then
      self:StartOrEndRaid()
   end
end

function MagicDKP:GetProfileParam(var) 
   local varName = var[#var]
   if self.spam then
      self:spam("Getting parameter %s as %s.", varName, tostring(db[varName]))
   end
   return db[varName]
end

function MagicDKP:UpdateItem(itemid, ignore, bank, value, type)
   itemid = tonumber(itemid)
   local item = items[itemid] or {} 
   item.ignore = ignore -- entirely ignore this item
   item.bank   = bank   -- always send to bank
   item.value  = value  -- default / fixed price
   item.type   = type   -- nil = ask, true = fixed
   item.itemid = itemid
   item.itemInfo = { GetItemInfo(itemid) }
 
   items[itemid] = item

   self:AddItemConfig(itemid)
   self:NotifyChange()
end

function MagicDKP:AddItemConfig(itemid)
   local ii = items[itemid].itemInfo
   local itemName = ii[1]
   local itemLink  = ii[2]
   local itemTexture = ii[10]
   local itemRarity = ii[3]
   if not GetItemInfo(itemid) then
      --      if self.trace then self:trace("Unknown item %d, loading via GameTooltip...", itemid) end
      items[itemid].name = items[itemid].name or itemName or "Unknown Item: "..itemid
      items[itemid].rarity = items[itemid].rarity or itemRarity or 100
      items[itemid].unknown = true
      if not itemLink then
	 itemLink = items[itemid].name
      end
   else
      items[itemid].name = itemName
      items[itemid].unknown = nil
      items[itemid].rarity = itemRarity
      items[itemid].texture = itemTexture
   end

   itemLink = gsub(gsub(itemLink, "[\[]", ""), "[\]]", "")

   local itemcfg = options.args.itemdb.args[tostring(itemid)] or {}

   itemcfg.type = "group"
   itemcfg.name = itemLink
   itemcfg.order = (20 - items[itemid].rarity) * 1000 + (itemLevel or 0)
   itemcfg.get = "GetOption"
   itemcfg.set = "SetOption"
   itemcfg.args = standardItemOptions

   options.args.itemdb.args[tostring(itemid)] = itemcfg
end


function MagicDKP:IsDefaultBossEvent(info)
   local mob, region  = tonumber(info[#info-1]), info[#info-2]
   local hash = zones[region].mobs[mob]
   local defHash =  MagicDKP.defaultZones[zones[region].name]
   if not defHash or not defHash.mobs[mob] then
      return not hash.event or hash.event == hash.name
   else
      return hash.event == defHash.mobs[mob].event
   end
end

function MagicDKP:ResetBossEvent(info, value)
   local mob, region  = tonumber(info[#info-1]), info[#info-2]
   local defHash = MagicDKP.defaultZones[zones[region].name]
   if not defHash or not defHash.mobs[mob] then
      zones[region].mobs[mob].event = nil
   else
      zones[region].mobs[mob].event = defHash.mobs[mob].event
   end
end

function MagicDKP:SetEventOption(info, value)
   local var = info[#info]
   local event = info[#info-1]
   local raid = tonumber(info[#info-2])
   if var == "value" then
      raids[raid].events[event].value = tonumber(value)
   end
end

function MagicDKP:GetEventOption(info)
   local var = info[#info]
   local event = info[#info-1]
   local raid = tonumber(info[#info-2])
   local value = raids[raid].events[event][var]
   if value then 
      return tostring(value)
   elseif var == "value" then
      return "0"
   end
end

function MagicDKP:SetLootOption(info, value)
   local var,num = string.match(info[#info], idmatch)
   local event = info[#info-2]
   local raid = tonumber(info[#info-3])
   num = tonumber(num)

   if self.spam then self:spam("|cffffff00SetLootOption:|r %d/%s/%d/%s => %s", raid, event, num, tostring(var), tostring(value)) end

   if var ~= "looter" then
      value = tonumber(value)
   elseif value == "00BANK" then
      value = "bank"
   elseif value == "00DE" then
      value = "disenchant"
   end
   
   raids[raid].events[event].items[num][var] = value
end

function MagicDKP:GetLootOption(info)
   local var,num = string.match(info[#info], idmatch)
   local event = info[#info-2]
   local raid = tonumber(info[#info-3])
   num = tonumber(num)
   local value = raids[raid].events[event].items[num][var]
   if self.spam then self:spam("GetLootOption: %d/%s/%d/%s => %s", raid, event, num, tostring(var), tostring(value)) end
   if value and var == "value" then 
      return tostring(value)
   elseif var == "looter" then
      if value == "bank" then
	 value = "00BANK"
      elseif value == "disenchant" then
	 value = "00DE"
      end
   end
   return value
end

function MagicDKP:SetBossOption(info, value, state)
   local var = info[#info]
   local mob = tonumber(info[#info-1])
   local region = info[#info-2]
   local old = zones[region].mobs[mob][var]
   if var == "event" and old then
      for id, data in pairs(zones[region].mobs) do
	 if data.event == old then
	    data.event = value
	    if self.spam then
	       self:spam("|cffffff00SetBossOption:|r %s/%d/%s => %s", region, id, var, tostring(value))
	    end
	 end
      end
   else 
      zones[region].mobs[mob][var] = value or nil
   end
   if self.spam then self:spam("|cffffff00SetBossOption:|r %s/%d/%s => %s", region, mob, var, tostring(value)) end
end

function MagicDKP:GetBossOption(info, key)
   local var = info[#info]
   local mob = tonumber(info[#info-1])
   local region = info[#info-2]
   local value = zones[region].mobs[mob][var]

   if var == "event" then
      if not value then
	 value  = zones[region].mobs[mob].name
      end
   end
--      value = PRI_LIST[value or 1]
--   elseif var == "category" then
--      value = ACT_LIST[value or 1]
   if self.spam then self:spam("GetBossOption: %s/%d/%s => %s", region, mob, var, tostring(value)) end
   return value 
end

function MagicDKP:IsItemIgnored(var)
   return items[tonumber(var[#var-1])].ignore
end

function MagicDKP:IsItemUnvalued(var)
   local item = items[tonumber(var[#var-1])]
   return item.ignore or item.bank
end

function MagicDKP:IsItemKnown(var)
   local item = items[tonumber(var[#var-1])]
   return not item.unknown
end

function MagicDKP:RefreshItemData(itemid)
   local name = GetItemData(itemid)
   if not name then
      self:ScheduleTimer("RefreshItemData", 0.5, tonumber(itemid))
      return
   end
   self:AddItemConfig(itemid)
   self:NotifyChange()
end
				  
				  
function MagicDKP:FetchItemData(var)
   local itemid = var[#var-1]
   GameTooltip:SetHyperlink("item:"..itemid)
   self:ScheduleTimer("RefreshItemData", 0.5, tonumber(itemid))
end

function MagicDKP:SetOption(info, value, state)
   local var = info[#info]
   local item = info[#info-1]
   local section = info[#info-2]
   if section == "itemdb" then
      if var == "value" then
	 items[tonumber(item)][var] = value and tonumber(value) or nil
      else
	 items[tonumber(item)][var] = value or nil
      end
   end
   if self.spam then self:spam("|cffffff00SetOption:|r %s/%s/%s => %s", section, item, var, tostring(value)) end
end

function MagicDKP:GetEventName(info)
   return events[info[#info-1]]
end

function MagicDKP:SetEventName(info, value)
   events[info[#info-1]] = value
end

function MagicDKP:GetOption(info, key)
   local var = info[#info]
   local item  = info[#info-1]
   local section = info[#info-2]
   local value
   if section == "itemdb" then
      value = items[tonumber(item)][var]
      if var == "value" then
	 value = tostring(value or 0) 
      end
   end
   if self.spam then self:spam("GetOption: %s/%s/%s => %s", section, item, var, tostring(value)) end
   return value or false
end

function MagicDKP:AddNewBoss(uid, name, zone)
   zone.mobs[uid] = {
      name = BR[name] or name
   }
   self:BuildZoneConfig(self:MakeKey(zone.name))
   if self.info then self:info("Detected new boss %s in zone %s.", name, zone.name) end
end

function MagicDKP:GetOptions()
   return options
end

function MagicDKP:GetZoneName(zone)
   local simple 
   if not zone then
      zone = GetRealZoneText()
   end
   zone = ZR[zone] or zone
   simple = self:MakeKey(zone)
   return simple, zone
end

LibStub("AceConfig-3.0"):RegisterOptionsTable(L["Magic DKP"],  function(name) return (name == "dialog" and options) or cmdoptions end,
					      { "magicdkp", "md", "mdkp" })

 
